"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class DataSourceProperties(str, Enum):
    r"""Names of data source fields supported for filtering/sorting on list endpoint."""

    NAME = "name"
    DESCRIPTION = "description"
    LABELS = "labels"
    CHUNK = "chunk"
    CHUNK_DELIMITER = "chunk_delimiter"
    UPDATED_AT = "updated_at"
    LAST_UPDATED_BY = "last_updated_by"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class StaticToolParameterType(str, Enum):
    r"""The expected type for a static tool parameter."""

    STRING = "string"
    INT = "int"
    BOOLEAN = "boolean"
    DATA_SOURCE_LIST = "data_source_list"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class InsightsProperties(str, Enum):
    ID = "id"
    SESSION_ID = "session_id"
    UPLOAD_FILE_ID = "upload_file_id"
    WORKFLOW_ID = "workflow_id"
    UPLOAD_FOLDER_ID = "upload_folder_id"
    INSIGHT_KEY = "insight_key"
    INSIGHT_TOOL_ID = "insight_tool_id"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import io
import pydantic
from syllable_sdk.types import BaseModel
from syllable_sdk.utils import FieldMetadata, MultipartFormMetadata
from typing import IO, Optional, Union
from typing_extensions import Annotated, NotRequired, TypedDict


class BodyInsightsFolderUploadFileFileTypedDict(TypedDict):
    file_name: str
    content: Union[bytes, IO[bytes], io.BufferedReader]
    content_type: NotRequired[str]


class BodyInsightsFolderUploadFileFile(BaseModel):
    file_name: Annotated[
        str, pydantic.Field(alias="fileName"), FieldMetadata(multipart=True)
    ]

    content: Annotated[
        Union[bytes, IO[bytes], io.BufferedReader],
        pydantic.Field(alias=""),
        FieldMetadata(multipart=MultipartFormMetadata(content=True)),
    ]

    content_type: Annotated[
        Optional[str],
        pydantic.Field(alias="Content-Type"),
        FieldMetadata(multipart=True),
    ] = None


class BodyInsightsFolderUploadFileTypedDict(TypedDict):
    file: NotRequired[BodyInsightsFolderUploadFileFileTypedDict]


class BodyInsightsFolderUploadFile(BaseModel):
    file: Annotated[
        Optional[BodyInsightsFolderUploadFileFile],
        FieldMetadata(multipart=MultipartFormMetadata(file=True)),
    ] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from syllable_sdk.types import BaseModel
from typing_extensions import TypedDict


class TwilioNumberUpdateResponseTypedDict(TypedDict):
    r"""Response model for updating a Twilio number and associating it with a channel."""

    phone_number: str
    r"""The updated Twilio phone number"""


class TwilioNumberUpdateResponse(BaseModel):
    r"""Response model for updating a Twilio number and associating it with a channel."""

    phone_number: str
    r"""The updated Twilio phone number"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .languagegroupproperties import LanguageGroupProperties
from .orderbydirection import OrderByDirection
from pydantic import model_serializer
from syllable_sdk.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from syllable_sdk.utils import FieldMetadata, QueryParamMetadata
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class LanguageGroupsListRequestTypedDict(TypedDict):
    page: NotRequired[Nullable[int]]
    r"""The page number from which to start (0-based)"""
    limit: NotRequired[int]
    r"""The maximum number of items to return"""
    search_fields: NotRequired[List[LanguageGroupProperties]]
    r"""String names of fields to search. Correspond by index to search field values"""
    search_field_values: NotRequired[List[str]]
    r"""Values of fields to search. Correspond by index to search fields. Unless field name contains \"list\", an individual search field value cannot be a list"""
    order_by: NotRequired[Nullable[LanguageGroupProperties]]
    r"""The field whose value should be used to order the results"""
    order_by_direction: NotRequired[Nullable[OrderByDirection]]
    r"""The direction in which to order the results"""
    fields: NotRequired[Nullable[List[LanguageGroupProperties]]]
    r"""The fields to include in the response"""
    start_datetime: NotRequired[Nullable[str]]
    r"""The start datetime for filtering results"""
    end_datetime: NotRequired[Nullable[str]]
    r"""The end datetime for filtering results"""


class LanguageGroupsListRequest(BaseModel):
    page: Annotated[
        OptionalNullable[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""The page number from which to start (0-based)"""

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 25
    r"""The maximum number of items to return"""

    search_fields: Annotated[
        Optional[List[LanguageGroupProperties]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""String names of fields to search. Correspond by index to search field values"""

    search_field_values: Annotated[
        Optional[List[str]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Values of fields to search. Correspond by index to search fields. Unless field name contains \"list\", an individual search field value cannot be a list"""

    order_by: Annotated[
        OptionalNullable[LanguageGroupProperties],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""The field whose value should be used to order the results"""

    order_by_direction: Annotated[
        OptionalNullable[OrderByDirection],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""The direction in which to order the results"""

    fields: Annotated[
        OptionalNullable[List[LanguageGroupProperties]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""The fields to include in the response"""

    start_datetime: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""The start datetime for filtering results"""

    end_datetime: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""The end datetime for filtering results"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "page",
            "limit",
            "search_fields",
            "search_field_values",
            "order_by",
            "order_by_direction",
            "fields",
            "start_datetime",
            "end_datetime",
        ]
        nullable_fields = [
            "page",
            "order_by",
            "order_by_direction",
            "fields",
            "start_datetime",
            "end_datetime",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from syllable_sdk.types import BaseModel
from typing_extensions import TypedDict


class TwilioChannelConfigCreateTypedDict(TypedDict):
    r"""Twilio channel config information."""

    account_sid: str
    r"""SID of the Twilio account"""
    auth_token: str
    r"""The Twilio auth token"""


class TwilioChannelConfigCreate(BaseModel):
    r"""Twilio channel config information."""

    account_sid: str
    r"""SID of the Twilio account"""

    auth_token: str
    r"""The Twilio auth token"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class DialogRole(str, Enum):
    r"""Role of dialog"""

    AGENT = "agent"
    USER = "user"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .twiliophonenumber import TwilioPhoneNumber, TwilioPhoneNumberTypedDict
from syllable_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class TwilioListNumbersResponseTypedDict(TypedDict):
    r"""Response model for a list of Twilio phone numbers"""

    phone_numbers: List[TwilioPhoneNumberTypedDict]
    r"""List of phone numbers."""


class TwilioListNumbersResponse(BaseModel):
    r"""Response model for a list of Twilio phone numbers"""

    phone_numbers: List[TwilioPhoneNumber]
    r"""List of phone numbers."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .httpclient import AsyncHttpClient, ClientOwner, HttpClient, close_clients
from .sdkconfiguration import SDKConfiguration
from .utils.logger import Logger, get_default_logger
from .utils.retries import RetryConfig
import httpx
import importlib
from syllable_sdk import models, utils
from syllable_sdk._hooks import SDKHooks
from syllable_sdk.types import OptionalNullable, UNSET
import sys
from typing import Any, Callable, Dict, Optional, TYPE_CHECKING, Union, cast
import weakref

if TYPE_CHECKING:
    from syllable_sdk.agents import Agents
    from syllable_sdk.channels import Channels
    from syllable_sdk.conversations import Conversations
    from syllable_sdk.custom_messages import CustomMessages
    from syllable_sdk.dashboards import Dashboards
    from syllable_sdk.data_sources import DataSources
    from syllable_sdk.events import Events
    from syllable_sdk.incidents import Incidents
    from syllable_sdk.insights_sdk import InsightsSDK
    from syllable_sdk.language_groups import LanguageGroups
    from syllable_sdk.organizations import Organizations
    from syllable_sdk.outbound import Outbound
    from syllable_sdk.permissions import Permissions
    from syllable_sdk.prompts import Prompts
    from syllable_sdk.roles import Roles
    from syllable_sdk.services import Services
    from syllable_sdk.session_debug import SessionDebug
    from syllable_sdk.session_labels import SessionLabels
    from syllable_sdk.sessions import Sessions
    from syllable_sdk.takeouts import Takeouts
    from syllable_sdk.tools import Tools
    from syllable_sdk.users import Users
    from syllable_sdk.v1 import V1


class SyllableSDK(BaseSDK):
    r"""SyllableSDK:
    # Syllable Platform SDK

    Syllable SDK gives you the power of awesome AI agentry. 🚀

    ## Overview

    The Syllable SDK provides a comprehensive set of tools and APIs to integrate powerful AI
    capabilities into your communication applications. Whether you're building phone agents, chatbots,
    virtual assistants, or any other AI-driven solutions, Syllable SDK has got you covered.

    ## Features

    - **Agent Configuration**: Create and manage agents that can interact with users across various
    channels.
    - **Channel Management**: Configure channels like SMS, web chat, and more to connect agents with
    users.
    - **Custom Messages**: Set up custom messages that agents can deliver as greetings or responses.
    - **Conversations**: Track and manage conversations between users and agents, including session
    management.
    - **Tools and Workflows**: Leverage tools and workflows to enhance agent capabilities, such as data
    processing and API calls.
    - **Data Sources**: Integrate data sources to provide agents with additional context and
    information.
    - **Insights and Analytics**: Analyze conversations and sessions to gain insights into user
    interactions.
    - **Permissions and Security**: Manage permissions to control access to various features and
    functionalities.
    - **Language Support**: Define language groups to enable multilingual support for agents.
    - **Outbound Campaigns**: Create and manage outbound communication campaigns to reach users
    effectively.
    - **Session Labels**: Label sessions with evaluations of quality and descriptions of issues
    encountered.
    - **Incident Management**: Track and manage incidents related to agent interactions.

    """

    agents: "Agents"
    r"""Operations related to agent configuration. When a user interacts with the           Syllable system, they do so by communicating with an agent.           An agent is linked to a prompt, a custom message, and one or more channel targets to           define its behavior and capabilities. For more information, see           [Console docs](https://docs.syllable.ai/workspaces/Agents)."""
    channels: "Channels"
    r"""Operations related to channel configuration.           A channel is an organization-level point of communication, like a phone number or a web           chat. A channel can be associated with an agent by creating a channel target linking           them."""
    conversations: "Conversations"
    r"""Operations related to conversations.           A conversation is a record of messages between a user and an agent, and is composed of           one or more sessions."""
    data_sources: "DataSources"
    r"""Operations related to data sources. A data source is a blob of text that           can be made available to an agent's general info tools to provide more context to the           agent when generating its responses. For more information, see           [Console docs](https://docs.syllable.ai/Resources/DataSources)."""
    events: "Events"
    r"""Operations related to events. An event represents a specific occurrence           during a session. Currently the API/SDK only supports fetching logged events."""
    incidents: "Incidents"
    r"""Operations related to incidents."""
    insights: "InsightsSDK"
    r"""Operations related to insights results. An insight is a tool that processes          conversation data to extract information and generate reports."""
    custom_messages: "CustomMessages"
    r"""Operations related to custom message configuration.           A custom message is a pre-configured message delivered by an agent as a greeting at the           beginning of a conversation. Multiple agents can use the same custom mesasage. A custom           message has one or more rules defined, which allow for different messages to be           dynamically selected and delivered at runtime based on the current time and either           date or day of the week. For more information, see           [Console docs](https://docs.syllable.ai/Resources/Messages)."""
    permissions: "Permissions"
    r"""Operations related to permissions. A permission is a specific           capability or access level granted to a user within the Syllable system.           Permissions are used to control access to various features and functionalities."""
    prompts: "Prompts"
    r"""Operations related to prompts. A prompt defines the behavior of an           agent by delivering instructions to the LLM about how the agent should behave.           A prompt can be linked to one or more agents. A prompt can also be linked to tools to           allow an agent using the prompt to use them. For more information, see           [Console docs](https://docs.syllable.ai/Resources/Prompts)."""
    roles: "Roles"
    r"""Operations related to roles. A role is a collection of permissions           that can be assigned to users to control their access to various features within the           Syllable system."""
    services: "Services"
    r"""Operations related to service configuration. A service is a collection of           tools. You can specify an authentication method and values on a service, and any linked           tools will use that auth information to generate headers for HTTP calls."""
    session_labels: "SessionLabels"
    r"""Operations related to labeling sessions with evaluations of quality and           descriptions of issues the user encountered or other details. For more information, see           [Console docs](https://docs.syllable.ai/workspaces/Sessions)."""
    sessions: "Sessions"
    r"""Operations related to sessions. A session is a building block of a           conversation. For more information, see           [Console docs](https://docs.syllable.ai/workspaces/Sessions)."""
    session_debug: "SessionDebug"
    tools: "Tools"
    r"""Operations related to tool configuration. A tool is a function that an           agent can call to perform actions like accessing databases, making API calls, or           processing data. For an agent to have access to a tool, the prompt associated with that           agent should be linked to the tool and include instructions to use it. For more           information, see [Console docs](https://docs.syllable.ai/Resources/Tools)."""
    dashboards: "Dashboards"
    r"""Operations related to dashboards. Currently the API/SDK           only supports fetching basic information about dashboards."""
    organizations: "Organizations"
    r"""Operations related to organizations."""
    outbound: "Outbound"
    language_groups: "LanguageGroups"
    r"""Operations related to language groups. A language group is a           collection of language, voice, and DTMF configuration that can be linked to an agent to           define the languages and voices it supports. For more information, see           [Console docs](https://docs.syllable.ai/Resources/LanguageGroups)."""
    takeouts: "Takeouts"
    users: "Users"
    v1: "V1"
    _sub_sdk_map = {
        "agents": ("syllable_sdk.agents", "Agents"),
        "channels": ("syllable_sdk.channels", "Channels"),
        "conversations": ("syllable_sdk.conversations", "Conversations"),
        "data_sources": ("syllable_sdk.data_sources", "DataSources"),
        "events": ("syllable_sdk.events", "Events"),
        "incidents": ("syllable_sdk.incidents", "Incidents"),
        "insights": ("syllable_sdk.insights_sdk", "InsightsSDK"),
        "custom_messages": ("syllable_sdk.custom_messages", "CustomMessages"),
        "permissions": ("syllable_sdk.permissions", "Permissions"),
        "prompts": ("syllable_sdk.prompts", "Prompts"),
        "roles": ("syllable_sdk.roles", "Roles"),
        "services": ("syllable_sdk.services", "Services"),
        "session_labels": ("syllable_sdk.session_labels", "SessionLabels"),
        "sessions": ("syllable_sdk.sessions", "Sessions"),
        "session_debug": ("syllable_sdk.session_debug", "SessionDebug"),
        "tools": ("syllable_sdk.tools", "Tools"),
        "dashboards": ("syllable_sdk.dashboards", "Dashboards"),
        "organizations": ("syllable_sdk.organizations", "Organizations"),
        "outbound": ("syllable_sdk.outbound", "Outbound"),
        "language_groups": ("syllable_sdk.language_groups", "LanguageGroups"),
        "takeouts": ("syllable_sdk.takeouts", "Takeouts"),
        "users": ("syllable_sdk.users", "Users"),
        "v1": ("syllable_sdk.v1", "V1"),
    }

    def __init__(
        self,
        api_key_header: Optional[
            Union[Optional[str], Callable[[], Optional[str]]]
        ] = None,
        server_idx: Optional[int] = None,
        server_url: Optional[str] = None,
        url_params: Optional[Dict[str, str]] = None,
        client: Optional[HttpClient] = None,
        async_client: Optional[AsyncHttpClient] = None,
        retry_config: OptionalNullable[RetryConfig] = UNSET,
        timeout_ms: Optional[int] = None,
        debug_logger: Optional[Logger] = None,
    ) -> None:
        r"""Instantiates the SDK configuring it with the provided parameters.

        :param api_key_header: The api_key_header required for authentication
        :param server_idx: The index of the server to use for all methods
        :param server_url: The server URL to use for all methods
        :param url_params: Parameters to optionally template the server URL with
        :param client: The HTTP client to use for all synchronous methods
        :param async_client: The Async HTTP client to use for all asynchronous methods
        :param retry_config: The retry configuration to use for all supported methods
        :param timeout_ms: Optional request timeout applied to each operation in milliseconds
        """
        client_supplied = True
        if client is None:
            client = httpx.Client()
            client_supplied = False

        assert issubclass(
            type(client), HttpClient
        ), "The provided client must implement the HttpClient protocol."

        async_client_supplied = True
        if async_client is None:
            async_client = httpx.AsyncClient()
            async_client_supplied = False

        if debug_logger is None:
            debug_logger = get_default_logger()

        assert issubclass(
            type(async_client), AsyncHttpClient
        ), "The provided async_client must implement the AsyncHttpClient protocol."

        security: Any = None
        if callable(api_key_header):
            # pylint: disable=unnecessary-lambda-assignment
            security = lambda: models.Security(api_key_header=api_key_header())
        else:
            security = models.Security(api_key_header=api_key_header)

        if server_url is not None:
            if url_params is not None:
                server_url = utils.template_url(server_url, url_params)

        BaseSDK.__init__(
            self,
            SDKConfiguration(
                client=client,
                client_supplied=client_supplied,
                async_client=async_client,
                async_client_supplied=async_client_supplied,
                security=security,
                server_url=server_url,
                server_idx=server_idx,
                retry_config=retry_config,
                timeout_ms=timeout_ms,
                debug_logger=debug_logger,
            ),
            parent_ref=self,
        )

        hooks = SDKHooks()

        # pylint: disable=protected-access
        self.sdk_configuration.__dict__["_hooks"] = hooks

        current_server_url, *_ = self.sdk_configuration.get_server_details()
        server_url, self.sdk_configuration.client = hooks.sdk_init(
            current_server_url, client
        )
        if current_server_url != server_url:
            self.sdk_configuration.server_url = server_url

        weakref.finalize(
            self,
            close_clients,
            cast(ClientOwner, self.sdk_configuration),
            self.sdk_configuration.client,
            self.sdk_configuration.client_supplied,
            self.sdk_configuration.async_client,
            self.sdk_configuration.async_client_supplied,
        )

    def dynamic_import(self, modname, retries=3):
        for attempt in range(retries):
            try:
                return importlib.import_module(modname)
            except KeyError:
                # Clear any half-initialized module and retry
                sys.modules.pop(modname, None)
                if attempt == retries - 1:
                    break
        raise KeyError(f"Failed to import module '{modname}' after {retries} attempts")

    def __getattr__(self, name: str):
        if name in self._sub_sdk_map:
            module_path, class_name = self._sub_sdk_map[name]
            try:
                module = self.dynamic_import(module_path)
                klass = getattr(module, class_name)
                instance = klass(self.sdk_configuration, parent_ref=self)
                setattr(self, name, instance)
                return instance
            except ImportError as e:
                raise AttributeError(
                    f"Failed to import module {module_path} for attribute {name}: {e}"
                ) from e
            except AttributeError as e:
                raise AttributeError(
                    f"Failed to find class {class_name} in module {module_path} for attribute {name}: {e}"
                ) from e

        raise AttributeError(
            f"'{type(self).__name__}' object has no attribute '{name}'"
        )

    def __dir__(self):
        default_attrs = list(super().__dir__())
        lazy_attrs = list(self._sub_sdk_map.keys())
        return sorted(list(set(default_attrs + lazy_attrs)))

    def __enter__(self):
        return self

    async def __aenter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if (
            self.sdk_configuration.client is not None
            and not self.sdk_configuration.client_supplied
        ):
            self.sdk_configuration.client.close()
        self.sdk_configuration.client = None

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if (
            self.sdk_configuration.async_client is not None
            and not self.sdk_configuration.async_client_supplied
        ):
            await self.sdk_configuration.async_client.aclose()
        self.sdk_configuration.async_client = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class EventProperties(str, Enum):
    TIMESTAMP = "timestamp"
    SESSION_ID = "session_id"
    CONVERSATION_ID = "conversation_id"
    SOURCE = "source"
    SOURCE_ID = "source_id"
    CATEGORY = "category"
    TYPE = "type"
    USER_ID = "user_id"
    DESCRIPTION = "description"
    ATTRIBUTES = "attributes"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class UserProperties(str, Enum):
    r"""Names of user fields supported for filtering/sorting on list endpoint."""

    EMAIL = "email"
    FIRST_NAME = "first_name"
    LAST_NAME = "last_name"
    ROLE_ID = "role_id"
    ROLE_NAME = "role_name"
    EMAIL_FIRST_NAME_LAST_NAME = "email_first_name_last_name"
    ACTIVITY_STATUS = "activity_status"
    LAST_UPDATED = "last_updated"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from datetime import datetime
from syllable_sdk import errors, models, utils
from syllable_sdk._hooks import HookContext
from syllable_sdk.types import BaseModel, OptionalNullable, UNSET
from syllable_sdk.utils import get_security_from_env
from syllable_sdk.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, List, Mapping, Optional, Union, cast


class Folders(BaseSDK):
    r"""Operations related to insights upload folders. An insight folder is used          to upload call recordings for insight workflow analysis."""

    def list(
        self,
        *,
        page: OptionalNullable[int] = UNSET,
        limit: Optional[int] = 25,
        search_fields: Optional[List[models.InsightsFolderProperties]] = None,
        search_field_values: Optional[List[str]] = None,
        order_by: OptionalNullable[models.InsightsFolderProperties] = UNSET,
        order_by_direction: OptionalNullable[models.OrderByDirection] = UNSET,
        fields: OptionalNullable[List[models.InsightsFolderProperties]] = UNSET,
        start_datetime: OptionalNullable[str] = UNSET,
        end_datetime: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ListResponseInsightsFolder:
        r"""List Insights Upload Folders

        :param page: The page number from which to start (0-based)
        :param limit: The maximum number of items to return
        :param search_fields: String names of fields to search. Correspond by index to search field values
        :param search_field_values: Values of fields to search. Correspond by index to search fields. Unless field name contains \"list\", an individual search field value cannot be a list
        :param order_by: The field whose value should be used to order the results
        :param order_by_direction: The direction in which to order the results
        :param fields: The fields to include in the response
        :param start_datetime: The start datetime for filtering results
        :param end_datetime: The end datetime for filtering results
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.InsightsFolderListRequest(
            page=page,
            limit=limit,
            search_fields=search_fields,
            search_field_values=search_field_values,
            order_by=order_by,
            order_by_direction=order_by_direction,
            fields=fields,
            start_datetime=start_datetime,
            end_datetime=end_datetime,
        )

        req = self._build_request(
            method="GET",
            path="/api/v1/insights/folders/",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="insights_folder_list",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.ListResponseInsightsFolder, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def list_async(
        self,
        *,
        page: OptionalNullable[int] = UNSET,
        limit: Optional[int] = 25,
        search_fields: Optional[List[models.InsightsFolderProperties]] = None,
        search_field_values: Optional[List[str]] = None,
        order_by: OptionalNullable[models.InsightsFolderProperties] = UNSET,
        order_by_direction: OptionalNullable[models.OrderByDirection] = UNSET,
        fields: OptionalNullable[List[models.InsightsFolderProperties]] = UNSET,
        start_datetime: OptionalNullable[str] = UNSET,
        end_datetime: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ListResponseInsightsFolder:
        r"""List Insights Upload Folders

        :param page: The page number from which to start (0-based)
        :param limit: The maximum number of items to return
        :param search_fields: String names of fields to search. Correspond by index to search field values
        :param search_field_values: Values of fields to search. Correspond by index to search fields. Unless field name contains \"list\", an individual search field value cannot be a list
        :param order_by: The field whose value should be used to order the results
        :param order_by_direction: The direction in which to order the results
        :param fields: The fields to include in the response
        :param start_datetime: The start datetime for filtering results
        :param end_datetime: The end datetime for filtering results
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.InsightsFolderListRequest(
            page=page,
            limit=limit,
            search_fields=search_fields,
            search_field_values=search_field_values,
            order_by=order_by,
            order_by_direction=order_by_direction,
            fields=fields,
            start_datetime=start_datetime,
            end_datetime=end_datetime,
        )

        req = self._build_request_async(
            method="GET",
            path="/api/v1/insights/folders/",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="insights_folder_list",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.ListResponseInsightsFolder, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def create(
        self,
        *,
        request: Union[models.InsightsFolderInput, models.InsightsFolderInputTypedDict],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.InsightsFolder:
        r"""Create Insights Upload Folder

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, models.InsightsFolderInput)
        request = cast(models.InsightsFolderInput, request)

        req = self._build_request(
            method="POST",
            path="/api/v1/insights/folders/",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.InsightsFolderInput
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="insights_folder_create",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.InsightsFolder, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def create_async(
        self,
        *,
        request: Union[models.InsightsFolderInput, models.InsightsFolderInputTypedDict],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.InsightsFolder:
        r"""Create Insights Upload Folder

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, models.InsightsFolderInput)
        request = cast(models.InsightsFolderInput, request)

        req = self._build_request_async(
            method="POST",
            path="/api/v1/insights/folders/",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.InsightsFolderInput
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="insights_folder_create",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.InsightsFolder, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def get_by_id(
        self,
        *,
        folder_id: int,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.FolderDetails:
        r"""Get Insights Folder Details

        :param folder_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.InsightsFolderGetByIDRequest(
            folder_id=folder_id,
        )

        req = self._build_request(
            method="GET",
            path="/api/v1/insights/folders/{folder_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="insights_folder_get_by_id",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.FolderDetails, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def get_by_id_async(
        self,
        *,
        folder_id: int,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.FolderDetails:
        r"""Get Insights Folder Details

        :param folder_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.InsightsFolderGetByIDRequest(
            folder_id=folder_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/api/v1/insights/folders/{folder_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="insights_folder_get_by_id",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.FolderDetails, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def delete(
        self,
        *,
        folder_id: int,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> bool:
        r"""Delete Insights Folder

        :param folder_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.InsightsFolderDeleteRequest(
            folder_id=folder_id,
        )

        req = self._build_request(
            method="DELETE",
            path="/api/v1/insights/folders/{folder_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="insights_folder_delete",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(bool, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def delete_async(
        self,
        *,
        folder_id: int,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> bool:
        r"""Delete Insights Folder

        :param folder_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.InsightsFolderDeleteRequest(
            folder_id=folder_id,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/api/v1/insights/folders/{folder_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="insights_folder_delete",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(bool, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def update(
        self,
        *,
        folder_id: int,
        insights_folder_input: Union[
            models.InsightsFolderInput, models.InsightsFolderInputTypedDict
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.InsightsFolder:
        r"""Update Insights Folder

        :param folder_id:
        :param insights_folder_input:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.InsightsFolderUpdateRequest(
            folder_id=folder_id,
            insights_folder_input=utils.get_pydantic_model(
                insights_folder_input, models.InsightsFolderInput
            ),
        )

        req = self._build_request(
            method="PUT",
            path="/api/v1/insights/folders/{folder_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.insights_folder_input,
                False,
                False,
                "json",
                models.InsightsFolderInput,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="insights_folder_update",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.InsightsFolder, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def update_async(
        self,
        *,
        folder_id: int,
        insights_folder_input: Union[
            models.InsightsFolderInput, models.InsightsFolderInputTypedDict
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.InsightsFolder:
        r"""Update Insights Folder

        :param folder_id:
        :param insights_folder_input:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.InsightsFolderUpdateRequest(
            folder_id=folder_id,
            insights_folder_input=utils.get_pydantic_model(
                insights_folder_input, models.InsightsFolderInput
            ),
        )

        req = self._build_request_async(
            method="PUT",
            path="/api/v1/insights/folders/{folder_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.insights_folder_input,
                False,
                False,
                "json",
                models.InsightsFolderInput,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="insights_folder_update",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.InsightsFolder, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def upload_file(
        self,
        *,
        folder_id: int,
        call_id: str,
        agent_number: OptionalNullable[str] = UNSET,
        customer_number: OptionalNullable[str] = UNSET,
        start_time: OptionalNullable[datetime] = UNSET,
        end_time: OptionalNullable[datetime] = UNSET,
        duration: OptionalNullable[float] = UNSET,
        metadata: OptionalNullable[str] = UNSET,
        body_insights_folder_upload_file: Optional[
            Union[
                models.BodyInsightsFolderUploadFile,
                models.BodyInsightsFolderUploadFileTypedDict,
            ]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.InsightsUploadFile:
        r"""Upload Insights  Upload Folder

        :param folder_id:
        :param call_id: A unique identifier for the call
        :param agent_number: The phone number or ID of the agent involved.
        :param customer_number: The phone number or ID of the customer.
        :param start_time: The timestamp of the call's beginning
        :param end_time: The timestamp of the call's end.
        :param duration: The call duration in seconds.
        :param metadata:
        :param body_insights_folder_upload_file:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.InsightsFolderUploadFileRequest(
            folder_id=folder_id,
            call_id=call_id,
            agent_number=agent_number,
            customer_number=customer_number,
            start_time=start_time,
            end_time=end_time,
            duration=duration,
            metadata=metadata,
            body_insights_folder_upload_file=utils.get_pydantic_model(
                body_insights_folder_upload_file,
                Optional[models.BodyInsightsFolderUploadFile],
            ),
        )

        req = self._build_request(
            method="POST",
            path="/api/v1/insights/folders/{folder_id}/upload-file",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body_insights_folder_upload_file,
                False,
                True,
                "multipart",
                Optional[models.BodyInsightsFolderUploadFile],
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="insights_folder_upload_file",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.InsightsUploadFile, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def upload_file_async(
        self,
        *,
        folder_id: int,
        call_id: str,
        agent_number: OptionalNullable[str] = UNSET,
        customer_number: OptionalNullable[str] = UNSET,
        start_time: OptionalNullable[datetime] = UNSET,
        end_time: OptionalNullable[datetime] = UNSET,
        duration: OptionalNullable[float] = UNSET,
        metadata: OptionalNullable[str] = UNSET,
        body_insights_folder_upload_file: Optional[
            Union[
                models.BodyInsightsFolderUploadFile,
                models.BodyInsightsFolderUploadFileTypedDict,
            ]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.InsightsUploadFile:
        r"""Upload Insights  Upload Folder

        :param folder_id:
        :param call_id: A unique identifier for the call
        :param agent_number: The phone number or ID of the agent involved.
        :param customer_number: The phone number or ID of the customer.
        :param start_time: The timestamp of the call's beginning
        :param end_time: The timestamp of the call's end.
        :param duration: The call duration in seconds.
        :param metadata:
        :param body_insights_folder_upload_file:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.InsightsFolderUploadFileRequest(
            folder_id=folder_id,
            call_id=call_id,
            agent_number=agent_number,
            customer_number=customer_number,
            start_time=start_time,
            end_time=end_time,
            duration=duration,
            metadata=metadata,
            body_insights_folder_upload_file=utils.get_pydantic_model(
                body_insights_folder_upload_file,
                Optional[models.BodyInsightsFolderUploadFile],
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/api/v1/insights/folders/{folder_id}/upload-file",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body_insights_folder_upload_file,
                False,
                True,
                "multipart",
                Optional[models.BodyInsightsFolderUploadFile],
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="insights_folder_upload_file",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.InsightsUploadFile, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def list_files(
        self,
        *,
        folder_id: int,
        page: OptionalNullable[int] = UNSET,
        limit: Optional[int] = 25,
        search_fields: Optional[List[models.InsightsUploadFileProperties]] = None,
        search_field_values: Optional[List[str]] = None,
        order_by: OptionalNullable[models.InsightsUploadFileProperties] = UNSET,
        order_by_direction: OptionalNullable[models.OrderByDirection] = UNSET,
        fields: OptionalNullable[List[models.InsightsUploadFileProperties]] = UNSET,
        start_datetime: OptionalNullable[str] = UNSET,
        end_datetime: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ListResponseInsightsUploadFile:
        r"""Fetch Insights Upload Files

        :param folder_id:
        :param page: The page number from which to start (0-based)
        :param limit: The maximum number of items to return
        :param search_fields: String names of fields to search. Correspond by index to search field values
        :param search_field_values: Values of fields to search. Correspond by index to search fields. Unless field name contains \"list\", an individual search field value cannot be a list
        :param order_by: The field whose value should be used to order the results
        :param order_by_direction: The direction in which to order the results
        :param fields: The fields to include in the response
        :param start_datetime: The start datetime for filtering results
        :param end_datetime: The end datetime for filtering results
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.InsightsUploadListFilesRequest(
            folder_id=folder_id,
            page=page,
            limit=limit,
            search_fields=search_fields,
            search_field_values=search_field_values,
            order_by=order_by,
            order_by_direction=order_by_direction,
            fields=fields,
            start_datetime=start_datetime,
            end_datetime=end_datetime,
        )

        req = self._build_request(
            method="GET",
            path="/api/v1/insights/folders/{folder_id}/files",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="insights_upload_list_files",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.ListResponseInsightsUploadFile, http_res
            )
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def list_files_async(
        self,
        *,
        folder_id: int,
        page: OptionalNullable[int] = UNSET,
        limit: Optional[int] = 25,
        search_fields: Optional[List[models.InsightsUploadFileProperties]] = None,
        search_field_values: Optional[List[str]] = None,
        order_by: OptionalNullable[models.InsightsUploadFileProperties] = UNSET,
        order_by_direction: OptionalNullable[models.OrderByDirection] = UNSET,
        fields: OptionalNullable[List[models.InsightsUploadFileProperties]] = UNSET,
        start_datetime: OptionalNullable[str] = UNSET,
        end_datetime: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ListResponseInsightsUploadFile:
        r"""Fetch Insights Upload Files

        :param folder_id:
        :param page: The page number from which to start (0-based)
        :param limit: The maximum number of items to return
        :param search_fields: String names of fields to search. Correspond by index to search field values
        :param search_field_values: Values of fields to search. Correspond by index to search fields. Unless field name contains \"list\", an individual search field value cannot be a list
        :param order_by: The field whose value should be used to order the results
        :param order_by_direction: The direction in which to order the results
        :param fields: The fields to include in the response
        :param start_datetime: The start datetime for filtering results
        :param end_datetime: The end datetime for filtering results
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.InsightsUploadListFilesRequest(
            folder_id=folder_id,
            page=page,
            limit=limit,
            search_fields=search_fields,
            search_field_values=search_field_values,
            order_by=order_by,
            order_by_direction=order_by_direction,
            fields=fields,
            start_datetime=start_datetime,
            end_datetime=end_datetime,
        )

        req = self._build_request_async(
            method="GET",
            path="/api/v1/insights/folders/{folder_id}/files",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="insights_upload_list_files",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.ListResponseInsightsUploadFile, http_res
            )
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def move_files(
        self,
        *,
        folder_id: int,
        insights_folder_file_move: Union[
            models.InsightsFolderFileMove, models.InsightsFolderFileMoveTypedDict
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[models.InsightsUploadFile]:
        r"""Move Insights Upload Files

        :param folder_id:
        :param insights_folder_file_move:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.InsightsUploadMoveFilesRequest(
            folder_id=folder_id,
            insights_folder_file_move=utils.get_pydantic_model(
                insights_folder_file_move, models.InsightsFolderFileMove
            ),
        )

        req = self._build_request(
            method="POST",
            path="/api/v1/insights/folders/{folder_id}/move-files",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.insights_folder_file_move,
                False,
                False,
                "json",
                models.InsightsFolderFileMove,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="insights_upload_move_files",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(List[models.InsightsUploadFile], http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def move_files_async(
        self,
        *,
        folder_id: int,
        insights_folder_file_move: Union[
            models.InsightsFolderFileMove, models.InsightsFolderFileMoveTypedDict
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[models.InsightsUploadFile]:
        r"""Move Insights Upload Files

        :param folder_id:
        :param insights_folder_file_move:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.InsightsUploadMoveFilesRequest(
            folder_id=folder_id,
            insights_folder_file_move=utils.get_pydantic_model(
                insights_folder_file_move, models.InsightsFolderFileMove
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/api/v1/insights/folders/{folder_id}/move-files",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.insights_folder_file_move,
                False,
                False,
                "json",
                models.InsightsFolderFileMove,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="insights_upload_move_files",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(List[models.InsightsUploadFile], http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class AgentWaitSound(str, Enum):
    r"""Wait sounds supported by agents."""

    NO_SOUND = "No Sound"
    KEYBOARD_1 = "Keyboard 1"
    KEYBOARD_2 = "Keyboard 2"
    CALL_CENTER = "Call Center"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .prompthistorylinkedtool import (
    PromptHistoryLinkedTool,
    PromptHistoryLinkedToolTypedDict,
)
from .promptllmconfig import PromptLlmConfig, PromptLlmConfigTypedDict
from datetime import datetime
from pydantic import model_serializer
from syllable_sdk.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class PromptHistoryTypedDict(TypedDict):
    r"""Record of a specific version of a prompt."""

    timestamp: datetime
    r"""Timestamp of the change resulting in this version"""
    prompt_id: str
    r"""ID of the prompt"""
    version_number: int
    r"""Version number of this version. Starts at 1 when prompt is created, and incremented on each change."""
    prompt_text: str
    r"""Text of the prompt at this version"""
    prompt_name: str
    r"""Name of the prompt at this version"""
    user_email: str
    r"""Email address of the user who made the change that resulted in this version"""
    is_pre_enhancements: bool
    r"""Whether this version of the prompt was created before history of tool-prompt linking was tracked"""
    prompt_description: NotRequired[Nullable[str]]
    r"""Description of the prompt at this version"""
    llm_config: NotRequired[Nullable[PromptLlmConfigTypedDict]]
    r"""String representation of LLM config for the prompt at this version"""
    comments: NotRequired[Nullable[str]]
    r"""Comments describing the change that resulted in this version"""
    linked_tools: NotRequired[List[PromptHistoryLinkedToolTypedDict]]
    r"""Tools that were linked to this version of the prompt"""


class PromptHistory(BaseModel):
    r"""Record of a specific version of a prompt."""

    timestamp: datetime
    r"""Timestamp of the change resulting in this version"""

    prompt_id: str
    r"""ID of the prompt"""

    version_number: int
    r"""Version number of this version. Starts at 1 when prompt is created, and incremented on each change."""

    prompt_text: str
    r"""Text of the prompt at this version"""

    prompt_name: str
    r"""Name of the prompt at this version"""

    user_email: str
    r"""Email address of the user who made the change that resulted in this version"""

    is_pre_enhancements: bool
    r"""Whether this version of the prompt was created before history of tool-prompt linking was tracked"""

    prompt_description: OptionalNullable[str] = UNSET
    r"""Description of the prompt at this version"""

    llm_config: OptionalNullable[PromptLlmConfig] = UNSET
    r"""String representation of LLM config for the prompt at this version"""

    comments: OptionalNullable[str] = UNSET
    r"""Comments describing the change that resulted in this version"""

    linked_tools: Optional[List[PromptHistoryLinkedTool]] = None
    r"""Tools that were linked to this version of the prompt"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "prompt_description",
            "llm_config",
            "comments",
            "linked_tools",
        ]
        nullable_fields = ["prompt_description", "llm_config", "comments"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from syllable_sdk.types import BaseModel
from typing_extensions import TypedDict


class DialogMessageTypedDict(TypedDict):
    text: str
    r"""Dialog message text"""


class DialogMessage(BaseModel):
    text: str
    r"""Dialog message text"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from syllable_sdk.types import BaseModel
from typing_extensions import TypedDict


class InsightWorkflowEstimateTypedDict(TypedDict):
    r"""Response model for an insight workflow."""

    backfill_count: int
    r"""Number of extant calls that will be processed by the workflow"""
    backfill_duration: float
    r"""Estimated duration of the calls that will be processed by the workflow"""
    estimated_daily_count: int
    r"""Estimated number of calls that will be processed by the workflow daily"""
    estimated_daily_duration: float
    r"""Estimated duration of the calls that will be processed by the workflow daily"""
    estimated_daily_cost: float
    r"""Estimated cost of the calls that will be processed by the workflow daily"""
    estimated_backfill_cost: float
    r"""Estimated cost of the backfill of the calls that will be processed by the workflow"""


class InsightWorkflowEstimate(BaseModel):
    r"""Response model for an insight workflow."""

    backfill_count: int
    r"""Number of extant calls that will be processed by the workflow"""

    backfill_duration: float
    r"""Estimated duration of the calls that will be processed by the workflow"""

    estimated_daily_count: int
    r"""Estimated number of calls that will be processed by the workflow daily"""

    estimated_daily_duration: float
    r"""Estimated duration of the calls that will be processed by the workflow daily"""

    estimated_daily_cost: float
    r"""Estimated cost of the calls that will be processed by the workflow daily"""

    estimated_backfill_cost: float
    r"""Estimated cost of the backfill of the calls that will be processed by the workflow"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .toolauthtype import ToolAuthType
from pydantic import model_serializer
from syllable_sdk.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from typing import Any
from typing_extensions import NotRequired, TypedDict


class ServiceUpdateRequestTypedDict(TypedDict):
    r"""Request model to update an existing service."""

    name: str
    r"""The name of the service"""
    description: str
    r"""The description of the service"""
    id: int
    r"""The internal ID of the service"""
    auth_type: NotRequired[Nullable[ToolAuthType]]
    r"""The type of authentication to use for the service's tools"""
    auth_values: NotRequired[Nullable[Any]]
    r"""The values to use for the authentication, as a dict. Should contain \"username\" and \"password\" keys if auth type is basic, \"token\" key if auth type is bearer, arbitrary header keys if auth type is custom_headers. or \"client_id\", \"client_secret\", and \"auth_url\" keys if auth type is oauth2. On an update, leave a value for a given key null and the value in the database will not be updated. (If a key is omitted entirely, any existing value for that key will be removed.)"""
    last_updated_comments: NotRequired[Nullable[str]]
    r"""Free text providing comment about what was updated"""


class ServiceUpdateRequest(BaseModel):
    r"""Request model to update an existing service."""

    name: str
    r"""The name of the service"""

    description: str
    r"""The description of the service"""

    id: int
    r"""The internal ID of the service"""

    auth_type: OptionalNullable[ToolAuthType] = UNSET
    r"""The type of authentication to use for the service's tools"""

    auth_values: OptionalNullable[Any] = UNSET
    r"""The values to use for the authentication, as a dict. Should contain \"username\" and \"password\" keys if auth type is basic, \"token\" key if auth type is bearer, arbitrary header keys if auth type is custom_headers. or \"client_id\", \"client_secret\", and \"auth_url\" keys if auth type is oauth2. On an update, leave a value for a given key null and the value in the database will not be updated. (If a key is omitted entirely, any existing value for that key will be removed.)"""

    last_updated_comments: OptionalNullable[str] = UNSET
    r"""Free text providing comment about what was updated"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["auth_type", "auth_values", "last_updated_comments"]
        nullable_fields = ["auth_type", "auth_values", "last_updated_comments"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from syllable_sdk.types import BaseModel
from typing_extensions import TypedDict


class ToolPromptInfoTypedDict(TypedDict):
    r"""Information about a prompt linked to a tool."""

    id: int
    r"""The ID of the prompt"""
    name: str
    r"""The name of the prompt"""


class ToolPromptInfo(BaseModel):
    r"""Information about a prompt linked to a tool."""

    id: int
    r"""The ID of the prompt"""

    name: str
    r"""The name of the prompt"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .insightsoutput import InsightsOutput, InsightsOutputTypedDict
from pydantic import model_serializer
from syllable_sdk.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from typing import List
from typing_extensions import NotRequired, TypedDict


class ListResponseInsightsOutputTypedDict(TypedDict):
    items: List[InsightsOutputTypedDict]
    r"""List of items returned from the query"""
    page: int
    r"""The page number of the results (0-based)"""
    page_size: int
    r"""The number of items returned per page"""
    total_pages: NotRequired[Nullable[int]]
    r"""The total number of pages of results given the indicated page size"""
    total_count: NotRequired[Nullable[int]]
    r"""The total number of items returned from the query"""


class ListResponseInsightsOutput(BaseModel):
    items: List[InsightsOutput]
    r"""List of items returned from the query"""

    page: int
    r"""The page number of the results (0-based)"""

    page_size: int
    r"""The number of items returned per page"""

    total_pages: OptionalNullable[int] = UNSET
    r"""The total number of pages of results given the indicated page size"""

    total_count: OptionalNullable[int] = UNSET
    r"""The total number of items returned from the query"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["total_pages", "total_count"]
        nullable_fields = ["total_pages", "total_count"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .promptllmconfig import PromptLlmConfig, PromptLlmConfigTypedDict
from pydantic import model_serializer
from syllable_sdk.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class PromptUpdateRequestTypedDict(TypedDict):
    r"""Request model to update an existing prompt."""

    name: str
    r"""The prompt name"""
    type: str
    r"""The type of the prompt"""
    llm_config: PromptLlmConfigTypedDict
    r"""LLM configuration for a prompt."""
    id: int
    r"""The internal ID of the prompt"""
    description: NotRequired[Nullable[str]]
    r"""The description of the prompt"""
    context: NotRequired[Nullable[str]]
    r"""The prompt text that will be sent to the LLM at the beginning of the conversation"""
    tools: NotRequired[List[str]]
    r"""Names of tools to which the prompt has access"""
    edit_comments: NotRequired[Nullable[str]]
    r"""The comments for the most recent edit to the prompt"""
    include_default_tools: NotRequired[bool]
    r"""Whether to include the default tools (`hangup`) in the list of tools for the prompt. If you remove one of the default tools from your prompt, you might want to disable this option so that the tool is not added again when updated."""


class PromptUpdateRequest(BaseModel):
    r"""Request model to update an existing prompt."""

    name: str
    r"""The prompt name"""

    type: str
    r"""The type of the prompt"""

    llm_config: PromptLlmConfig
    r"""LLM configuration for a prompt."""

    id: int
    r"""The internal ID of the prompt"""

    description: OptionalNullable[str] = UNSET
    r"""The description of the prompt"""

    context: OptionalNullable[str] = UNSET
    r"""The prompt text that will be sent to the LLM at the beginning of the conversation"""

    tools: Optional[List[str]] = None
    r"""Names of tools to which the prompt has access"""

    edit_comments: OptionalNullable[str] = UNSET
    r"""The comments for the most recent edit to the prompt"""

    include_default_tools: Optional[bool] = True
    r"""Whether to include the default tools (`hangup`) in the list of tools for the prompt. If you remove one of the default tools from your prompt, you might want to disable this option so that the tool is not added again when updated."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "description",
            "context",
            "tools",
            "edit_comments",
            "include_default_tools",
        ]
        nullable_fields = ["description", "context", "edit_comments"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import io
import pydantic
from pydantic import model_serializer
from syllable_sdk.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from syllable_sdk.utils import FieldMetadata, MultipartFormMetadata
from typing import IO, Optional, Union
from typing_extensions import Annotated, NotRequired, TypedDict


class BodyOrganizationsUpdateLogoTypedDict(TypedDict):
    file_name: str
    content: Union[bytes, IO[bytes], io.BufferedReader]
    content_type: NotRequired[str]


class BodyOrganizationsUpdateLogo(BaseModel):
    file_name: Annotated[
        str, pydantic.Field(alias="fileName"), FieldMetadata(multipart=True)
    ]

    content: Annotated[
        Union[bytes, IO[bytes], io.BufferedReader],
        pydantic.Field(alias=""),
        FieldMetadata(multipart=MultipartFormMetadata(content=True)),
    ]

    content_type: Annotated[
        Optional[str],
        pydantic.Field(alias="Content-Type"),
        FieldMetadata(multipart=True),
    ] = None


class BodyOrganizationsUpdateTypedDict(TypedDict):
    display_name: str
    r"""The human-readable display name of the organization"""
    logo: NotRequired[BodyOrganizationsUpdateLogoTypedDict]
    r"""The organization logo image file to upload. Must be a PNG file and 120x120 pixels. If not provided, the logo will not be updated."""
    description: NotRequired[Nullable[str]]
    r"""Description of the organization"""
    domains: NotRequired[Nullable[str]]
    r"""Comma-delimited list of domains that users at the organization may have in their email addresses"""
    update_comments: NotRequired[Nullable[str]]
    r"""Comments about the update"""


class BodyOrganizationsUpdate(BaseModel):
    display_name: Annotated[str, FieldMetadata(multipart=True)]
    r"""The human-readable display name of the organization"""

    logo: Annotated[
        Optional[BodyOrganizationsUpdateLogo],
        FieldMetadata(multipart=MultipartFormMetadata(file=True)),
    ] = None
    r"""The organization logo image file to upload. Must be a PNG file and 120x120 pixels. If not provided, the logo will not be updated."""

    description: Annotated[OptionalNullable[str], FieldMetadata(multipart=True)] = UNSET
    r"""Description of the organization"""

    domains: Annotated[OptionalNullable[str], FieldMetadata(multipart=True)] = UNSET
    r"""Comma-delimited list of domains that users at the organization may have in their email addresses"""

    update_comments: Annotated[OptionalNullable[str], FieldMetadata(multipart=True)] = (
        UNSET
    )
    r"""Comments about the update"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["logo", "description", "domains", "update_comments"]
        nullable_fields = ["description", "domains", "update_comments"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from pydantic import model_serializer
from syllable_sdk.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from typing_extensions import NotRequired, TypedDict


class AttributesTypedDict(TypedDict):
    pass


class Attributes(BaseModel):
    pass


class EventTypedDict(TypedDict):
    r"""An event represents a specific occurrence during a session."""

    timestamp: datetime
    r"""Timestamp at which the event occurred"""
    session_id: NotRequired[Nullable[str]]
    r"""ID of the session in which the event occurred"""
    conversation_id: NotRequired[Nullable[str]]
    r"""ID of the conversation associated with the session in which the event occurred"""
    source: NotRequired[Nullable[str]]
    r"""Source of the event (either a tool call or an LLM prompt)"""
    source_id: NotRequired[Nullable[str]]
    r"""ID of the source of the event (either a tool ID or a prompt ID)"""
    category: NotRequired[Nullable[str]]
    r"""Broad categorization of the event"""
    type: NotRequired[Nullable[str]]
    r"""More specific class of the event"""
    user_id: NotRequired[Nullable[str]]
    r"""ID of the user interacting with the system when the event occurred (e.g., their       phone number)"""
    description: NotRequired[Nullable[str]]
    r"""Human-readable description of the event"""
    attributes: NotRequired[Nullable[AttributesTypedDict]]
    r"""Arbitrary additional metadata for the event"""


class Event(BaseModel):
    r"""An event represents a specific occurrence during a session."""

    timestamp: datetime
    r"""Timestamp at which the event occurred"""

    session_id: OptionalNullable[str] = UNSET
    r"""ID of the session in which the event occurred"""

    conversation_id: OptionalNullable[str] = UNSET
    r"""ID of the conversation associated with the session in which the event occurred"""

    source: OptionalNullable[str] = UNSET
    r"""Source of the event (either a tool call or an LLM prompt)"""

    source_id: OptionalNullable[str] = UNSET
    r"""ID of the source of the event (either a tool ID or a prompt ID)"""

    category: OptionalNullable[str] = UNSET
    r"""Broad categorization of the event"""

    type: OptionalNullable[str] = UNSET
    r"""More specific class of the event"""

    user_id: OptionalNullable[str] = UNSET
    r"""ID of the user interacting with the system when the event occurred (e.g., their       phone number)"""

    description: OptionalNullable[str] = UNSET
    r"""Human-readable description of the event"""

    attributes: OptionalNullable[Attributes] = UNSET
    r"""Arbitrary additional metadata for the event"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "session_id",
            "conversation_id",
            "source",
            "source_id",
            "category",
            "type",
            "user_id",
            "description",
            "attributes",
        ]
        nullable_fields = [
            "session_id",
            "conversation_id",
            "source",
            "source_id",
            "category",
            "type",
            "user_id",
            "description",
            "attributes",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from syllable_sdk.types import BaseModel
from typing import Dict
from typing_extensions import TypedDict


class CommunicationRequestTypedDict(TypedDict):
    reference_id: str
    r"""ID for target outreach (unique within batch)"""
    target: str
    r"""Target phone number"""
    request_variables: Dict[str, str]
    r"""Variables for request"""


class CommunicationRequest(BaseModel):
    reference_id: str
    r"""ID for target outreach (unique within batch)"""

    target: str
    r"""Target phone number"""

    request_variables: Dict[str, str]
    r"""Variables for request"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class DaysOfWeek(str, Enum):
    r"""Enum representing days of the week."""

    MON = "mon"
    TUE = "tue"
    WED = "wed"
    THU = "thu"
    FRI = "fri"
    SAT = "sat"
    SUN = "sun"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .toolargumentlocation import ToolArgumentLocation
from .toolhttpmethod import ToolHTTPMethod
from syllable_sdk.types import BaseModel
from typing_extensions import TypedDict


class ToolHTTPEndpointTypedDict(TypedDict):
    r"""The configuration for an HTTP API call by a tool."""

    url: str
    r"""The endpoint URL of the external service to call."""
    method: ToolHTTPMethod
    r"""The HTTP method to use for a tool HTTP request."""
    argument_location: ToolArgumentLocation
    r"""The location of the argument in a tool HTTP request.
    'body' is used for JSON data in the POST request body.
    'form' is used for form data in the POST request body.
    'path' is used for URL path parameters.
    'query' is used for query parameters in the URL.
    """


class ToolHTTPEndpoint(BaseModel):
    r"""The configuration for an HTTP API call by a tool."""

    url: str
    r"""The endpoint URL of the external service to call."""

    method: ToolHTTPMethod
    r"""The HTTP method to use for a tool HTTP request."""

    argument_location: ToolArgumentLocation
    r"""The location of the argument in a tool HTTP request.
    'body' is used for JSON data in the POST request body.
    'form' is used for form data in the POST request body.
    'path' is used for URL path parameters.
    'query' is used for query parameters in the URL.
    """



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class BatchStatus(str, Enum):
    r"""Status of a communication batch."""

    PENDING = "PENDING"
    ACTIVE = "ACTIVE"
    PAUSED = "PAUSED"
    FAILED = "FAILED"
    CANCELED = "CANCELED"
    EXPIRED = "EXPIRED"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from syllable_sdk.types import BaseModel
from typing_extensions import TypedDict


class TwilioNumberAddResponseTypedDict(TypedDict):
    r"""Response model for purchasing a Twilio number and associating it with a channel."""

    phone_number: str
    r"""The purchased Twilio phone number"""


class TwilioNumberAddResponse(BaseModel):
    r"""Response model for purchasing a Twilio number and associating it with a channel."""

    phone_number: str
    r"""The purchased Twilio phone number"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .agentlanguage import AgentLanguage, AgentLanguageTypedDict
from .agentvoicedisplayname import AgentVoiceDisplayName
from .agentvoicegender import AgentVoiceGender
from .agentvoicemodel import AgentVoiceModel
from .agentvoicevarname import AgentVoiceVarName
from .ttsprovider import TtsProvider
from syllable_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class AgentVoiceTypedDict(TypedDict):
    r"""Voice option for an agent."""

    provider: TtsProvider
    r"""TTS provider for an agent voice."""
    display_name: AgentVoiceDisplayName
    r"""Display names of voices that Syllable supports."""
    var_name: AgentVoiceVarName
    r"""The variable name of an agent voice (used when processing messages)."""
    gender: AgentVoiceGender
    r"""Gender for an agent voice."""
    model: AgentVoiceModel
    r"""Model for an agent voice."""
    supported_languages: List[AgentLanguageTypedDict]
    r"""Languages supported by the voice"""
    deprecated: bool
    r"""Whether the voice is deprecated and should not be used"""


class AgentVoice(BaseModel):
    r"""Voice option for an agent."""

    provider: TtsProvider
    r"""TTS provider for an agent voice."""

    display_name: AgentVoiceDisplayName
    r"""Display names of voices that Syllable supports."""

    var_name: AgentVoiceVarName
    r"""The variable name of an agent voice (used when processing messages)."""

    gender: AgentVoiceGender
    r"""Gender for an agent voice."""

    model: AgentVoiceModel
    r"""Model for an agent voice."""

    supported_languages: List[AgentLanguage]
    r"""Languages supported by the voice"""

    deprecated: bool
    r"""Whether the voice is deprecated and should not be used"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from pydantic import model_serializer
from syllable_sdk.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from typing import Dict, List, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


SampleRateTypedDict = TypeAliasType("SampleRateTypedDict", Union[int, float])
r"""Sample rate as a percentage of calls"""


SampleRate = TypeAliasType("SampleRate", Union[int, float])
r"""Sample rate as a percentage of calls"""


AgentListTypedDict = TypeAliasType("AgentListTypedDict", Union[List[int], List[str]])
r"""List of agents"""


AgentList = TypeAliasType("AgentList", Union[List[int], List[str]])
r"""List of agents"""


class InsightWorkflowConditionTypedDict(TypedDict):
    r"""Model for the conditions that trigger an insight workflow."""

    min_duration: NotRequired[Nullable[int]]
    r"""Minimum duration of the calls in seconds"""
    max_duration: NotRequired[Nullable[int]]
    r"""Maximum duration of the calls in seconds"""
    sample_rate: NotRequired[Nullable[SampleRateTypedDict]]
    r"""Sample rate as a percentage of calls"""
    agent_list: NotRequired[Nullable[AgentListTypedDict]]
    r"""List of agents"""
    prompt_list: NotRequired[Nullable[List[str]]]
    r"""List of prompts"""
    folder_list: NotRequired[Nullable[List[int]]]
    r"""List of folder IDs"""
    sheet_info: NotRequired[Nullable[Dict[str, str]]]
    r"""Information about the sheet to be used for the workflow"""


class InsightWorkflowCondition(BaseModel):
    r"""Model for the conditions that trigger an insight workflow."""

    min_duration: OptionalNullable[int] = UNSET
    r"""Minimum duration of the calls in seconds"""

    max_duration: OptionalNullable[int] = UNSET
    r"""Maximum duration of the calls in seconds"""

    sample_rate: OptionalNullable[SampleRate] = UNSET
    r"""Sample rate as a percentage of calls"""

    agent_list: OptionalNullable[AgentList] = UNSET
    r"""List of agents"""

    prompt_list: OptionalNullable[List[str]] = UNSET
    r"""List of prompts"""

    folder_list: OptionalNullable[List[int]] = UNSET
    r"""List of folder IDs"""

    sheet_info: OptionalNullable[Dict[str, str]] = UNSET
    r"""Information about the sheet to be used for the workflow"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "min_duration",
            "max_duration",
            "sample_rate",
            "agent_list",
            "prompt_list",
            "folder_list",
            "sheet_info",
        ]
        nullable_fields = [
            "min_duration",
            "max_duration",
            "sample_rate",
            "agent_list",
            "prompt_list",
            "folder_list",
            "sheet_info",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from pydantic import model_serializer
from syllable_sdk.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from typing import List
from typing_extensions import NotRequired, TypedDict


class SessionLabelTypedDict(TypedDict):
    r"""Response model for session label operations.
    A session label is associated with a given session and contains an evaluation of quality and
    descriptions of issues the user encountered in that session or other details.
    """

    session_id: int
    r"""The internal ID of the session (see Session.session_id)"""
    type: str
    r"""The type of the label (either \"auto-rating\" or \"human-rating\")"""
    code: str
    r"""A code describing the quality of the labeled session (either \"GOOD\", \"OK\", \"BAD\", or \"N/A\")"""
    user_email: str
    r"""The email of the user who created the label"""
    id: int
    r"""The internal ID of the label"""
    timestamp: str
    r"""The timestamp at which the label was created"""
    comments: NotRequired[Nullable[str]]
    r"""Comment string describing additional details about the session"""
    issue_categories: NotRequired[Nullable[List[str]]]
    r"""Descriptions of issues occurring in the labeled call"""


class SessionLabel(BaseModel):
    r"""Response model for session label operations.
    A session label is associated with a given session and contains an evaluation of quality and
    descriptions of issues the user encountered in that session or other details.
    """

    session_id: int
    r"""The internal ID of the session (see Session.session_id)"""

    type: str
    r"""The type of the label (either \"auto-rating\" or \"human-rating\")"""

    code: str
    r"""A code describing the quality of the labeled session (either \"GOOD\", \"OK\", \"BAD\", or \"N/A\")"""

    user_email: str
    r"""The email of the user who created the label"""

    id: int
    r"""The internal ID of the label"""

    timestamp: str
    r"""The timestamp at which the label was created"""

    comments: OptionalNullable[str] = UNSET
    r"""Comment string describing additional details about the session"""

    issue_categories: OptionalNullable[List[str]] = UNSET
    r"""Descriptions of issues occurring in the labeled call"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["comments", "issue_categories"]
        nullable_fields = ["comments", "issue_categories"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class CampaignProperties(str, Enum):
    ID = "id"
    CAMPAIGN_NAME = "campaign_name"
    CAMPAIGN_VARIABLES = "campaign_variables"
    DAILY_START_TIME = "daily_start_time"
    DAILY_END_TIME = "daily_end_time"
    SOURCE = "source"
    CALLER_ID = "caller_id"
    UPDATED_AT = "updated_at"
    LABEL = "label"
    LABELS = "labels"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from pydantic import model_serializer
from syllable_sdk.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from typing_extensions import NotRequired, TypedDict


class OrganizationResponseTypedDict(TypedDict):
    display_name: str
    r"""The human-readable display name of the organization."""
    id: int
    r"""The internal ID of the organization."""
    slug: str
    r"""The slug of the organization used for URLs in the Console UI"""
    last_updated: datetime
    r"""The timestamp of the most recent update to the organization"""
    description: NotRequired[Nullable[str]]
    r"""Description of the organization."""
    domains: NotRequired[Nullable[str]]
    r"""Comma-delimited list of domains that users at the organization may have in their email addresses."""
    last_updated_comments: NotRequired[Nullable[str]]
    r"""Comments for the most recent edit to the organization."""
    last_updated_by: NotRequired[Nullable[str]]
    r"""The email of the user who most recently updated the organization"""
    logo_url: NotRequired[Nullable[str]]
    r"""CDN URL. The org will always have a logo, but this value will be null on a response to an update where no logo was provided on the request"""


class OrganizationResponse(BaseModel):
    display_name: str
    r"""The human-readable display name of the organization."""

    id: int
    r"""The internal ID of the organization."""

    slug: str
    r"""The slug of the organization used for URLs in the Console UI"""

    last_updated: datetime
    r"""The timestamp of the most recent update to the organization"""

    description: OptionalNullable[str] = UNSET
    r"""Description of the organization."""

    domains: OptionalNullable[str] = UNSET
    r"""Comma-delimited list of domains that users at the organization may have in their email addresses."""

    last_updated_comments: OptionalNullable[str] = UNSET
    r"""Comments for the most recent edit to the organization."""

    last_updated_by: OptionalNullable[str] = UNSET
    r"""The email of the user who most recently updated the organization"""

    logo_url: OptionalNullable[str] = UNSET
    r"""CDN URL. The org will always have a logo, but this value will be null on a response to an update where no logo was provided on the request"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "description",
            "domains",
            "last_updated_comments",
            "last_updated_by",
            "logo_url",
        ]
        nullable_fields = [
            "description",
            "domains",
            "last_updated_comments",
            "last_updated_by",
            "logo_url",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from syllable_sdk.types import BaseModel
from typing_extensions import TypedDict


class LanguageGroupAgentInfoTypedDict(TypedDict):
    r"""Information about an agent linked to a language group."""

    id: int
    r"""The ID of the agent"""
    name: str
    r"""The name of the agent"""


class LanguageGroupAgentInfo(BaseModel):
    r"""Information about an agent linked to a language group."""

    id: int
    r"""The ID of the agent"""

    name: str
    r"""The name of the agent"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class LoginType(str, Enum):
    r"""The type of login to use for the user."""

    GOOGLE = "google"
    USERNAME_AND_PASSWORD = "username_and_password"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from syllable_sdk.types import BaseModel
from typing import Any
from typing_extensions import TypedDict


class InsightToolDefinitionTypedDict(TypedDict):
    r"""Model for an insight tool definition. This is a template that can be used by multiple insight
    tool configurations, each providing their own parameter values.
    """

    id: int
    r"""Unique ID for insight tool definition"""
    name: str
    r"""Human-readable name of insight tool definition"""
    type: str
    r"""Type of insight tool definition"""
    description: str
    r"""Text description of insight tool definition"""
    tool_parameters: Any
    r"""Parameters for tools that use this definition and their associated types"""
    tool_result_set: Any
    r"""Result key/types for insight tool definition"""


class InsightToolDefinition(BaseModel):
    r"""Model for an insight tool definition. This is a template that can be used by multiple insight
    tool configurations, each providing their own parameter values.
    """

    id: int
    r"""Unique ID for insight tool definition"""

    name: str
    r"""Human-readable name of insight tool definition"""

    type: str
    r"""Type of insight tool definition"""

    description: str
    r"""Text description of insight tool definition"""

    tool_parameters: Any
    r"""Parameters for tools that use this definition and their associated types"""

    tool_result_set: Any
    r"""Result key/types for insight tool definition"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .promptllmconfig import PromptLlmConfig, PromptLlmConfigTypedDict
from pydantic import model_serializer
from syllable_sdk.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class PromptCreateRequestTypedDict(TypedDict):
    r"""Request model to create a prompt."""

    name: str
    r"""The prompt name"""
    type: str
    r"""The type of the prompt"""
    llm_config: PromptLlmConfigTypedDict
    r"""LLM configuration for a prompt."""
    description: NotRequired[Nullable[str]]
    r"""The description of the prompt"""
    context: NotRequired[Nullable[str]]
    r"""The prompt text that will be sent to the LLM at the beginning of the conversation"""
    tools: NotRequired[List[str]]
    r"""Names of tools to which the prompt has access"""
    include_default_tools: NotRequired[bool]
    r"""Whether to include the default tools (`hangup`) in the list of tools for the prompt. If you disable this during creation, you might want to disable it during updates as well, otherwise the default tools will be added when updating the prompt."""


class PromptCreateRequest(BaseModel):
    r"""Request model to create a prompt."""

    name: str
    r"""The prompt name"""

    type: str
    r"""The type of the prompt"""

    llm_config: PromptLlmConfig
    r"""LLM configuration for a prompt."""

    description: OptionalNullable[str] = UNSET
    r"""The description of the prompt"""

    context: OptionalNullable[str] = UNSET
    r"""The prompt text that will be sent to the LLM at the beginning of the conversation"""

    tools: Optional[List[str]] = None
    r"""Names of tools to which the prompt has access"""

    include_default_tools: Optional[bool] = True
    r"""Whether to include the default tools (`hangup`) in the list of tools for the prompt. If you disable this during creation, you might want to disable it during updates as well, otherwise the default tools will be added when updating the prompt."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["description", "context", "tools", "include_default_tools"]
        nullable_fields = ["description", "context"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .insighttooldefinition import InsightToolDefinition, InsightToolDefinitionTypedDict
from datetime import datetime
from pydantic import model_serializer
from syllable_sdk.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from typing import Any, Optional
from typing_extensions import NotRequired, TypedDict


class InsightToolOutputTypedDict(TypedDict):
    r"""Response model for an insight tool configuration."""

    name: str
    r"""Human readable name of insight tool"""
    description: str
    r"""Text description of insight tool configuration"""
    version: int
    r"""Version of insight tool"""
    tool_arguments: Any
    r"""Arguments for calling the insight tool"""
    insight_tool_definition_id: int
    r"""Unique ID for insight tool definition used by this tool configuration"""
    id: int
    r"""Unique ID for insight tool"""
    last_updated_by: str
    r"""Email of user who last updated insight tool configuration"""
    insight_tool_definition: NotRequired[Nullable[InsightToolDefinitionTypedDict]]
    r"""Insight Tool Definition"""
    created_at: NotRequired[datetime]
    r"""Timestamp of at which insight tool configuration was created"""
    updated_at: NotRequired[datetime]
    r"""Timestamp at which insight tool configuration was last updated"""


class InsightToolOutput(BaseModel):
    r"""Response model for an insight tool configuration."""

    name: str
    r"""Human readable name of insight tool"""

    description: str
    r"""Text description of insight tool configuration"""

    version: int
    r"""Version of insight tool"""

    tool_arguments: Any
    r"""Arguments for calling the insight tool"""

    insight_tool_definition_id: int
    r"""Unique ID for insight tool definition used by this tool configuration"""

    id: int
    r"""Unique ID for insight tool"""

    last_updated_by: str
    r"""Email of user who last updated insight tool configuration"""

    insight_tool_definition: OptionalNullable[InsightToolDefinition] = UNSET
    r"""Insight Tool Definition"""

    created_at: Optional[datetime] = None
    r"""Timestamp of at which insight tool configuration was created"""

    updated_at: Optional[datetime] = None
    r"""Timestamp at which insight tool configuration was last updated"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["insight_tool_definition", "created_at", "updated_at"]
        nullable_fields = ["insight_tool_definition"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class AgentVoiceDisplayName(str, Enum):
    r"""Display names of voices that Syllable supports."""

    ACHERNAR_ENGLISH_ = "Achernar (English)"
    ACHERNAR_KOREAN_ = "Achernar (Korean)"
    ACHERNAR_MANDARIN_ = "Achernar (Mandarin)"
    ACHERNAR_SPANISH_ = "Achernar (Spanish)"
    ACHERNAR_THAI_ = "Achernar (Thai)"
    ACHERNAR_VIETNAMESE_ = "Achernar (Vietnamese)"
    ALICE = "Alice"
    ALLOY = "Alloy"
    AOEDE_ENGLISH_ = "Aoede (English)"
    AOEDE_KOREAN_ = "Aoede (Korean)"
    AOEDE_MANDARIN_ = "Aoede (Mandarin)"
    AOEDE_SPANISH_ = "Aoede (Spanish)"
    AOEDE_THAI_ = "Aoede (Thai)"
    AOEDE_VIETNAMESE_ = "Aoede (Vietnamese)"
    ASH = "Ash"
    BILL = "Bill"
    BRIAN = "Brian"
    CALLIRRHOE_ENGLISH_ = "Callirrhoe (English)"
    CALLIRRHOE_KOREAN_ = "Callirrhoe (Korean)"
    CALLIRRHOE_MANDARIN_ = "Callirrhoe (Mandarin)"
    CALLIRRHOE_SPANISH_ = "Callirrhoe (Spanish)"
    CALLIRRHOE_THAI_ = "Callirrhoe (Thai)"
    CALLIRRHOE_VIETNAMESE_ = "Callirrhoe (Vietnamese)"
    CALLUM = "Callum"
    CHARLIE = "Charlie"
    CHARLOTTE = "Charlotte"
    CHARON_ENGLISH_ = "Charon (English)"
    CHARON_KOREAN_ = "Charon (Korean)"
    CHARON_MANDARIN_ = "Charon (Mandarin)"
    CHARON_SPANISH_ = "Charon (Spanish)"
    CHARON_THAI_ = "Charon (Thai)"
    CHARON_VIETNAMESE_ = "Charon (Vietnamese)"
    CHRIS = "Chris"
    CMN_TW_WAVENET_A = "cmn-TW-Wavenet-A"
    CORAL = "Coral"
    DANIEL = "Daniel"
    ECHO = "Echo"
    EN_US_NEURAL2_D = "en-US-Neural2-D"
    EN_US_NEURAL2_F = "en-US-Neural2-F"
    EN_US_NEURAL2_J = "en-US-Neural2-J"
    EN_US_STUDIO_O = "en-US-Studio-O"
    ERIC = "Eric"
    ES_US_NEURAL2_A = "es-US-Neural2-A"
    ES_US_NEURAL2_B = "es-US-Neural2-B"
    FABLE = "Fable"
    FENRIR_ENGLISH_ = "Fenrir (English)"
    FENRIR_KOREAN_ = "Fenrir (Korean)"
    FENRIR_MANDARIN_ = "Fenrir (Mandarin)"
    FENRIR_SPANISH_ = "Fenrir (Spanish)"
    FENRIR_THAI_ = "Fenrir (Thai)"
    FENRIR_VIETNAMESE_ = "Fenrir (Vietnamese)"
    GEORGE = "George"
    JESSICA = "Jessica"
    KO_KR_NEURAL2_A = "ko-KR-Neural2-A"
    KORE_ENGLISH_ = "Kore (English)"
    KORE_KOREAN_ = "Kore (Korean)"
    KORE_MANDARIN_ = "Kore (Mandarin)"
    KORE_SPANISH_ = "Kore (Spanish)"
    KORE_THAI_ = "Kore (Thai)"
    KORE_VIETNAMESE_ = "Kore (Vietnamese)"
    LAURA = "Laura"
    LEDA_ENGLISH_ = "Leda (English)"
    LEDA_KOREAN_ = "Leda (Korean)"
    LEDA_MANDARIN_ = "Leda (Mandarin)"
    LEDA_SPANISH_ = "Leda (Spanish)"
    LEDA_THAI_ = "Leda (Thai)"
    LEDA_VIETNAMESE_ = "Leda (Vietnamese)"
    LIAM = "Liam"
    LILY = "Lily"
    MATILDA = "Matilda"
    NOVA = "Nova"
    ONYX = "Onyx"
    ORUS_ENGLISH_ = "Orus (English)"
    ORUS_KOREAN_ = "Orus (Korean)"
    ORUS_MANDARIN_ = "Orus (Mandarin)"
    ORUS_SPANISH_ = "Orus (Spanish)"
    ORUS_THAI_ = "Orus (Thai)"
    ORUS_VIETNAMESE_ = "Orus (Vietnamese)"
    PUCK_ENGLISH_ = "Puck (English)"
    PUCK_KOREAN_ = "Puck (Korean)"
    PUCK_MANDARIN_ = "Puck (Mandarin)"
    PUCK_SPANISH_ = "Puck (Spanish)"
    PUCK_THAI_ = "Puck (Thai)"
    PUCK_VIETNAMESE_ = "Puck (Vietnamese)"
    RIVER = "River"
    ROGER = "Roger"
    SARAH = "Sarah"
    SAGE = "Sage"
    SHIMMER = "Shimmer"
    UMBRIEL_ENGLISH_ = "Umbriel (English)"
    UMBRIEL_KOREAN_ = "Umbriel (Korean)"
    UMBRIEL_MANDARIN_ = "Umbriel (Mandarin)"
    UMBRIEL_SPANISH_ = "Umbriel (Spanish)"
    UMBRIEL_THAI_ = "Umbriel (Thai)"
    UMBRIEL_VIETNAMESE_ = "Umbriel (Vietnamese)"
    VINDEMIATRIX_ENGLISH_ = "Vindemiatrix (English)"
    VINDEMIATRIX_KOREAN_ = "Vindemiatrix (Korean)"
    VINDEMIATRIX_MANDARIN_ = "Vindemiatrix (Mandarin)"
    VINDEMIATRIX_SPANISH_ = "Vindemiatrix (Spanish)"
    VINDEMIATRIX_THAI_ = "Vindemiatrix (Thai)"
    VINDEMIATRIX_VIETNAMESE_ = "Vindemiatrix (Vietnamese)"
    VI_VN_NEURAL2_A = "vi-VN-Neural2-A"
    WILL = "Will"
    YUE_HK_STANDARD_C = "yue-HK-Standard-C"
    ZEPHYR_ENGLISH_ = "Zephyr (English)"
    ZEPHYR_KOREAN_ = "Zephyr (Korean)"
    ZEPHYR_MANDARIN_ = "Zephyr (Mandarin)"
    ZEPHYR_SPANISH_ = "Zephyr (Spanish)"
    ZEPHYR_THAI_ = "Zephyr (Thai)"
    ZEPHYR_VIETNAMESE_ = "Zephyr (Vietnamese)"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class SessionProperties(str, Enum):
    r"""Names of session fields supported for filtering/sorting on list endpoint."""

    TIMESTAMP = "timestamp"
    SESSION_ID = "session_id"
    CONVERSATION_ID = "conversation_id"
    CHANNEL_MANAGER_SERVICE = "channel_manager_service"
    CHANNEL_MANAGER_TYPE = "channel_manager_type"
    CHANNEL_MANAGER_SID = "channel_manager_sid"
    AGENT_TYPE = "agent_type"
    AGENT_ID = "agent_id"
    AGENT_NAME = "agent_name"
    PROMPT_ID = "prompt_id"
    PROMPT_NAME = "prompt_name"
    SOURCE = "source"
    TARGET = "target"
    DURATION = "duration"
    IS_LEGACY = "is_legacy"
    IS_TEST = "is_test"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from pydantic import model_serializer
from syllable_sdk.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from typing import Dict, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


JSONValueTypedDict = TypeAliasType("JSONValueTypedDict", Union[str, int, float])


JSONValue = TypeAliasType("JSONValue", Union[str, int, float])


class InsightsOutputTypedDict(TypedDict):
    r"""Response model for an insight tool."""

    id: int
    r"""Unique ID of the insight"""
    insight_tool_id: int
    r"""Unique ID for insight tool configuration"""
    insight_tool_version: int
    r"""Version of insight tool configuration"""
    insight_key: str
    r"""Key for insight tool result"""
    json_value: Dict[str, JSONValueTypedDict]
    r"""JSON value of insight tool result"""
    session_id: NotRequired[Nullable[int]]
    r"""Unique ID for the session"""
    upload_file_id: NotRequired[Nullable[int]]
    r"""Unique ID for uploaded file"""
    string_value: NotRequired[Nullable[str]]
    r"""String value of insight tool result"""
    numeric_value: NotRequired[Nullable[float]]
    r"""Numeric value of insight tool result"""
    created_at: NotRequired[datetime]
    r"""Timestamp at which insight tool result was created"""
    updated_at: NotRequired[datetime]
    r"""Timestamp at which insight tool result was last updated"""


class InsightsOutput(BaseModel):
    r"""Response model for an insight tool."""

    id: int
    r"""Unique ID of the insight"""

    insight_tool_id: int
    r"""Unique ID for insight tool configuration"""

    insight_tool_version: int
    r"""Version of insight tool configuration"""

    insight_key: str
    r"""Key for insight tool result"""

    json_value: Dict[str, JSONValue]
    r"""JSON value of insight tool result"""

    session_id: OptionalNullable[int] = UNSET
    r"""Unique ID for the session"""

    upload_file_id: OptionalNullable[int] = UNSET
    r"""Unique ID for uploaded file"""

    string_value: OptionalNullable[str] = UNSET
    r"""String value of insight tool result"""

    numeric_value: OptionalNullable[float] = UNSET
    r"""Numeric value of insight tool result"""

    created_at: Optional[datetime] = None
    r"""Timestamp at which insight tool result was created"""

    updated_at: Optional[datetime] = None
    r"""Timestamp at which insight tool result was last updated"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "session_id",
            "upload_file_id",
            "string_value",
            "numeric_value",
            "created_at",
            "updated_at",
        ]
        nullable_fields = [
            "session_id",
            "upload_file_id",
            "string_value",
            "numeric_value",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class BatchProperties(str, Enum):
    BATCH_ID = "batch_id"
    CAMPAIGN_ID = "campaign_id"
    STATUS = "status"
    EXPIRES_ON = "expires_on"
    LAST_UPDATED_AT = "last_updated_at"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .twiliochannelconfigcreate import (
    TwilioChannelConfigCreate,
    TwilioChannelConfigCreateTypedDict,
)
from syllable_sdk.types import BaseModel
from typing_extensions import TypedDict


class TwilioChannelCreateRequestTypedDict(TypedDict):
    r"""Request model for creating a Twilio channel."""

    name: str
    r"""The channel name"""
    config: TwilioChannelConfigCreateTypedDict
    r"""Twilio channel config information."""


class TwilioChannelCreateRequest(BaseModel):
    r"""Request model for creating a Twilio channel."""

    name: str
    r"""The channel name"""

    config: TwilioChannelConfigCreate
    r"""Twilio channel config information."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class AgentProperties(str, Enum):
    r"""Names of agent fields supported for filtering/sorting on list endpoint."""

    ID = "id"
    NAME = "name"
    NAME_EXACT = "name_exact"
    DESCRIPTION = "description"
    LABEL = "label"
    LABELS = "labels"
    NAME_DESCRIPTION = "name_description"
    TYPE = "type"
    TIMEZONE = "timezone"
    PROMPT_ID = "prompt_id"
    PROMPT_ID_LIST = "prompt_id_list"
    CUSTOM_MESSAGE_ID = "custom_message_id"
    LANGUAGES = "languages"
    VARIABLES = "variables"
    PROMPT_TOOL_DEFAULTS = "prompt_tool_defaults"
    TOOL_HEADERS = "tool_headers"
    UPDATED_AT = "updated_at"
    LAST_UPDATED_BY = "last_updated_by"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .body_insights_folder_upload_file import (
    BodyInsightsFolderUploadFile,
    BodyInsightsFolderUploadFileTypedDict,
)
from datetime import datetime
from pydantic import model_serializer
from syllable_sdk.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from syllable_sdk.utils import (
    FieldMetadata,
    PathParamMetadata,
    QueryParamMetadata,
    RequestMetadata,
)
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class InsightsFolderUploadFileRequestTypedDict(TypedDict):
    folder_id: int
    call_id: str
    r"""A unique identifier for the call"""
    agent_number: NotRequired[Nullable[str]]
    r"""The phone number or ID of the agent involved."""
    customer_number: NotRequired[Nullable[str]]
    r"""The phone number or ID of the customer."""
    start_time: NotRequired[Nullable[datetime]]
    r"""The timestamp of the call's beginning"""
    end_time: NotRequired[Nullable[datetime]]
    r"""The timestamp of the call's end."""
    duration: NotRequired[Nullable[float]]
    r"""The call duration in seconds."""
    metadata: NotRequired[Nullable[str]]
    body_insights_folder_upload_file: NotRequired[BodyInsightsFolderUploadFileTypedDict]


class InsightsFolderUploadFileRequest(BaseModel):
    folder_id: Annotated[
        int, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]

    call_id: Annotated[
        str, FieldMetadata(query=QueryParamMetadata(style="form", explode=True))
    ]
    r"""A unique identifier for the call"""

    agent_number: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""The phone number or ID of the agent involved."""

    customer_number: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""The phone number or ID of the customer."""

    start_time: Annotated[
        OptionalNullable[datetime],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""The timestamp of the call's beginning"""

    end_time: Annotated[
        OptionalNullable[datetime],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""The timestamp of the call's end."""

    duration: Annotated[
        OptionalNullable[float],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""The call duration in seconds."""

    metadata: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET

    body_insights_folder_upload_file: Annotated[
        Optional[BodyInsightsFolderUploadFile],
        FieldMetadata(request=RequestMetadata(media_type="multipart/form-data")),
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "agent_number",
            "customer_number",
            "start_time",
            "end_time",
            "duration",
            "metadata",
            "Body_insights_folder_upload_file",
        ]
        nullable_fields = [
            "agent_number",
            "customer_number",
            "start_time",
            "end_time",
            "duration",
            "metadata",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from syllable_sdk.types import BaseModel
from typing_extensions import TypedDict


class TwilioNumberAddRequestTypedDict(TypedDict):
    r"""Request model for purchasing a Twilio number and associating it with a channel."""

    friendly_name: str
    r"""Friendly name for the Twilio number"""
    area_code: str
    r"""Area code for the Twilio number"""


class TwilioNumberAddRequest(BaseModel):
    r"""Request model for purchasing a Twilio number and associating it with a channel."""

    friendly_name: str
    r"""Friendly name for the Twilio number"""

    area_code: str
    r"""Area code for the Twilio number"""
